@@ -1,6 +1,9 @@
 import pyproj
+from RWTHColors import ColorManager
 from ipyleaflet import Icon, TileLayer
 
+# Colors
+cm = ColorManager()
 
 # Projections
 proj = pyproj.Proj(proj='utm', zone=32, ellps='WGS84', preserve_units=True)
