@@ -1,22 +1,24 @@
 import io
 import itertools
 import logging
-from typing import Tuple, List, Union
+from typing import Tuple, List, Union, Optional
 
+import numpy as np
 from ipyleaflet import Map as LeafletMap, ScaleControl, FullScreenControl, Polyline, Icon, Marker, GeoData, \
     LayersControl, basemaps, basemap_to_tiles, Circle, LayerGroup
 from ipywidgets import HTML
 
 from pyridy.file import RDYFile
-from pyridy.osm.utils.elements import OSMRailwaySignal, OSMRailwaySwitch, OSMLevelCrossing, OSMResultNode, OSMRailwayElement
+from pyridy.osm.utils.elements import OSMRailwaySignal, OSMRailwaySwitch, OSMLevelCrossing, OSMResultNode, \
+    OSMRailwayElement
 from pyridy.utils import GPSSeries
 from pyridy import config
 
 from typing import TYPE_CHECKING
+
 if TYPE_CHECKING:
     from pyridy.campaign import Campaign
 
-
 logger = logging.getLogger(__name__)
 
 
@@ -74,31 +76,38 @@ class Map(LeafletMap):
         measurement_layers = []
 
         for file in campaign.files:
+            if len(file.measurements[GPSSeries]) == 0:
+                logger.warning(f"Coordinates are empty in file: {file.filename}")
+                continue
+
             track = create_measurement_layer(file)
             self.add(track)
             measurement_layers.append(track)
 
         return measurement_layers
 
-    def add_measurement(self, file: RDYFile) -> None:
+    def add_measurement(self, file: RDYFile, t_lim: Tuple[np.datetime64, np.datetime64] = None,
+                        show_hor_acc: bool = False) -> Optional[LayerGroup]:
         """ Adds a GPS track from a file to the Map
 
         Parameters
         ----------
-        campaign: Campaign
-            pyridy Campaign
-        name: str
-            Name of the file that should be drawn onto the map
+        t_lim: tuple, default: None
+            Time limit as a tuple of np.datetime64 to show only parts of the GPS track that are within the specified
+            time interval
         file: RDYFile
             Alternatively, provide RDYFile that should be drawn on the map
-
+        show_hor_acc : bool, default: False
+            If true shows the horizontal accuracies for each measurement point using circles. The likelihood that
+            that the real position is within the circle is defined as 68 %
         Returns
         -------
         The measurement layer
 
         """
-        measurement_layer = create_measurement_layer(file)
+        measurement_layer = create_measurement_layer(file, t_lim, show_hor_acc)
         measurement_layer.name = file.filename
+
         self.add(measurement_layer)
         self.refresh_controls()
         return measurement_layer
@@ -220,14 +229,29 @@ def create_marker(pos: Tuple[float, float], popup: Union[str, HTML] = None, colo
     return marker
 
 
-def create_measurement_layer(file: RDYFile):
-
+def create_measurement_layer(file: RDYFile, t_lim: Tuple[np.datetime64, np.datetime64] = None,
+                             show_hor_acc: bool = False) -> Optional[LayerGroup]:
     gps_series = file.measurements[GPSSeries]
     coords = gps_series.to_ipyleaflef()
+    time = gps_series.time
+    hor_acc = gps_series.hor_acc
 
     if not coords:
         logger.warning(f"Coordinates are empty in file: {file.filename}")
-        return
+        return None
+
+    if t_lim:
+        if type(t_lim) != tuple:
+            raise ValueError("t_lim must be a tuple of np.datetime64")
+
+        if t_lim[0] > t_lim[1]:
+            raise ValueError("The first datetime for t_lim must be smaller than the second!")
+
+        mask = (gps_series.time >= t_lim[0]) & (gps_series.time <= t_lim[1])
+
+        coords = [c for i, c in enumerate(coords) if mask[i]]
+        time = [t for i, t in enumerate(gps_series.time) if mask[i]]
+        hor_acc = [h for i, h in enumerate(gps_series.hor_acc) if mask[i]]
 
     measurement_layer = LayerGroup()
     measurement_layer.name = file.filename
@@ -235,13 +259,19 @@ def create_measurement_layer(file: RDYFile):
     file_polyline = Polyline(locations=coords, color=file.color, fill=False, weight=4,
                              dash_array='10, 10')
     file_message = HTML()
-    file_message.value = f"<p>{str(file.filename or '')}<br>{str(getattr(file.device, 'manufacturer', ''))}<br>{str(getattr(file.device, 'model', ''))}</p>"
+    file_message.value = f"<p>Filename: {str(file.filename or '')}" \
+                         f"<br>Device: {str(getattr(file.device, 'manufacturer', ''))} " \
+                         f"{str(getattr(file.device, 'model', ''))}</p>" + \
+                         f"<br>Start time: {str(time[0] or 'n/a')}" + \
+                         f"<br>End time:   {str(time[-1] or 'n/a')}"
+
     file_polyline.popup = file_message
     measurement_layer.add(file_polyline)
 
     # Add Start/End markers
     start_message = HTML()
     end_message = HTML()
+
     start_message.value = "<p>Start:</p>" + file_message.value
     end_message.value = "<p>End:</p>" + file_message.value
 
@@ -251,4 +281,20 @@ def create_measurement_layer(file: RDYFile):
     measurement_layer.add(start_marker)
     measurement_layer.add(end_marker)
 
+    if show_hor_acc:
+        circles = []
+        for c, h in zip(coords, hor_acc):
+            circle = Circle()
+            circle.location = (c[0], c[1])
+            circle.radius = int(h)
+            circle.color = "#00549F"
+            circle.fill_color = "#00549F"
+            circle.weight = 3
+            circle.fill_opacity = 0.1
+
+            circles.append(circle)
+
+        l_circles = LayerGroup(layers=circles)
+        measurement_layer.add(l_circles)
+
     return measurement_layer
